<?php
// Copyright (C) 2010-2012 Combodo SARL
//
//   This file is part of TeemIp.
//
//   TeemIp is free software; you can redistribute it and/or modify	
//   it under the terms of the GNU Affero General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   TeemIp is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU Affero General Public License for more details.
//
//   You should have received a copy of the GNU Affero General Public License
//   along with TeemIp. If not, see <http://www.gnu.org/licenses/>

/**
 * @copyright   Copyright (C) 2010-2012 Combodo SARL
 * @license     http://opensource.org/licenses/AGPL-3.0
 */

/*******************
 * Global constants
 */

define('MAX_IPV4_VALUE', 4294967295);
define('BLOCK_MIN_SIZE', 256);
define('SUBNET_MAX_SIZE', 65536);

define('ALL_ORGS', 65536);

define('ACTION_NONE', 0);
define('ACTION_SHRINK', 1);
define('ACTION_SPLIT', 2);
define('ACTION_EXPAND', 3);
define('ACTION_PARENT_BLOCK_IS_DELETED', 4);
define('ACTION_BLOCK_IS_DELETED', 5);

define('GLOBAL_CONFIG_DEFAULT_NAME', 'IPv4 Settings');
define('DEFAULT_BLOCK_LOW_WATER_MARK', 60);
define('DEFAULT_BLOCK_HIGH_WATER_MARK', 80);
define('DEFAULT_SUBNET_LOW_WATER_MARK', 60);
define('DEFAULT_SUBNET_HIGH_WATER_MARK', 80);
define('DEFAULT_IPRANGE_LOW_WATER_MARK', 60);
define('DEFAULT_IPRANGE_HIGH_WATER_MARK', 80);

define('DEFAULT_MAX_FREE_SPACE_OFFERS', 10);
define('DEFAULT_MAX_FREE_IP_OFFERS', 10);
define('DEFAULT_MAX_FREE_IP_OFFERS_WITH_PING', 5);
define('DEFAULT_SUBNET_CREATE_MAX_OFFER', 10);

define('RED', "#cc3300");
define('YELLOW', "#ffff00");
define('GREEN', "#33ff00");

define('TIME_TO_WAIT_FOR_PING_LONG', 3);
define('TIME_TO_WAIT_FOR_PING_SHORT', 1);
define('NUMBER_OF_PINGS', 1);
define('FAIL_KEY_FOR_PING', '100%');

define('NETWORK_IP_CODE', 'Network IP');
define('NETWORK_IP_DESC', 'Subnet IP');
define('BROADCAST_IP_CODE', 'Broadcast');
define('BROADCAST_IP_DESC', 'Broadcast IP');

/*****************************
 * Raw IPv4 Address attribute
 */

class AttributeRawIPAddress extends AttributeInteger
{
	protected function GetSQLCol() {return "CHAR(10)";}
	
	public function MakeRealValue($proposedValue, $oHostObj)
	{
		// Translate doted format into 10 characters prepended string used for storage
		// Translate only if entry is a dotes string IP. This avoids multiple re-entries in function.
		$sPattern = $this->GetValidationPattern(); 		
		if (preg_match ("/$sPattern/", $proposedValue))
		{
			$iValue = ip2long($proposedValue); 
			$sValue = sprintf("%010u", $iValue);
			//print_r ("prop: $proposedValue ip: $iValue sprintf: $sValue");
			return(($proposedValue == '255.255.255.255') ? MAX_IPV4_VALUE : sprintf("%010u", ip2long($proposedValue)));
		}
		else
		{
			return $proposedValue;
		}
	}
	                
	public function GetAsCSV($sValue, $sSeparator = ',', $sTextQualifier = '"', $oHostObject = null, $bLocalize = true)
	{
		//$sEscaped = (string)mylong2ip($sValue);
		$sEscaped = (string)$sValue;
		return $sTextQualifier.$sEscaped.$sTextQualifier;
	}
					
	public function GetAsHTML($sValue, $oHostObject = null, $bLocalize = true)
	{
		return Str::pure2html(mylong2ip($sValue));
	}

	public function GetAsXML($sValue, $oHostObject = null, $bLocalize = true)
	{
		//return Str::pure2xml((string)mylong2ip($sValue));
		return Str::pure2xml((string)$sValue);
	}

	public function GetEditValue($sAttCode, $oHostObject = null)
	{
		return mylong2ip($sAttCode);
	}
	
	public function GetValidationPattern()
	{
		$sNum = '(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])';
		return "^($sNum\\.$sNum\\.$sNum\\.$sNum)$";
	}
}

/**********************
 * Host Name Attribute
 */

class AttributeHostName extends AttributeString
{
	public function GetValidationPattern()
	{
		// By default, pattern matches RFC 1123 plus '_'
		return('^(\d|[a-z]|[A-Z]|_)(\d|[a-z]|[A-Z]|-|_)*$');
	}
}

/************************
 * Domain Name Attribute
 */

class AttributeDomainName extends AttributeString
{
	public function GetValidationPattern() 
	{
		// By default, pattern matches RFC 1123 plus '_'
		return('^(\d|[a-z]|[A-Z]|-|_)+((\.(\d|[a-z]|[A-Z]|-|_)+)*)\.?$');
	}
}

/************************
 * MAC Address Attribute
 */

class AttributeMacAddress extends AttributeString
{
	public function MakeRealValue($proposedValue, $oHostObj)
	{
		// Translate input value in canonical format used for storage
		// Input value = hyphens (12-34-56-78-90-ab), dots (1234.5678.90ab) or colons (12:34:56:78:90:ab)
		// Canonical Format = colons
		if ($proposedValue != '')
		{
			if ($proposedValue[2] == '-')
			{
				return(strtr($proposedValue, '-', ':'));
			}
			if ($proposedValue[4] == '.')
			{
				$proposedValue = str_replace('.', '', $proposedValue);
				$sOutputMac = '';
				$j = 0;
				for ($i = 0; $i < 12; $i++)
				{
					$sOutputMac[$i + $j] = $proposedValue[$i];
					if (($i > 0) && (is_int(($i - 1)/2)) && ($j < 5))
					{
						$j++;
						$sOutputMac[$i + $j] = ':';
					}
				}
				return(implode('',$sOutputMac));
			}
		}
		return ($proposedValue);
	}
	
	protected function GetMacAtFormat($sMac, $oHostObject)
	{
		// Return $sMac at format set by global parameters
		if (($sMac != '') && ($oHostObject != null))
		{
			$sMacAddressOutputFormat = $oHostObject->GetAttributeParams($this->GetCode());
			switch($sMacAddressOutputFormat)
			{
				case 'hyphens':
				// Return hyphens format
				return(strtr($sMac, ':', '-'));
				
				case 'dots':
				// Return dots format
				$sMac = str_replace(':', '', $sMac);
				$sOutputMac = '';
				$j = 0;
				for ($i = 0; $i < 12; $i++)
				{
					$sOutputMac[$i + $j] = $sMac[$i];
					if (($i == 3) || ($i == 7))
					{
						$j++;
						$sOutputMac[$i + $j] = '.';
					}
				}
				return(implode('',$sOutputMac));
				
				case 'colons':
				default:
				break;
			}
		}
		// Return default = registered = colons format
		return($sMac);
	}
	
	public function GetAsCSV($sValue, $sSeparator = ',', $sTextQualifier = '"', $oHostObject = null, $bLocalize = true)
	{
		$sFrom = array("\r\n", $sTextQualifier);
		$sTo = array("\n", $sTextQualifier.$sTextQualifier);
		$sEscaped = str_replace($sFrom, $sTo, (string)$this->GetMacAtFormat($sValue, $oHostObject));
		return $sTextQualifier.$sEscaped.$sTextQualifier;
	}

	public function GetAsHTML($sValue, $oHostObject = null, $bLocalize = true)
	{
		return Str::pure2html((string)$this->GetMacAtFormat($sValue, $oHostObject));
	}

	public function GetAsXML($sValue, $oHostObject = null, $bLocalize = true)
	{
		// XML being used by programs, we return canonical value of MAC 
		return Str::pure2xml((string)$sValue);
	}

	public function GetEditValue($sAttCode, $oHostObject = null)
	{
		return (string)$this->GetMacAtFormat($sAttCode, $oHostObject);
	}
	
	public function GetValidationPattern()
	{
		// By default, all 3 official pattern (colons, hyphens, dots) are accepted as input
		return('^((\d|([a-f]|[A-F])){2}-){5}(\d|([a-f]|[A-F])){2}$|^((\d|([a-f]|[A-F])){4}.){2}(\d|([a-f]|[A-F])){4}$|^((\d|([a-f]|[A-F])){2}:){5}(\d|([a-f]|[A-F])){2}$');
	}
}

/***********************
 * Percentage Attribute
 */

class AttributeIPPercentage extends AttributeInteger
{
	public function GetAsHTML($sValue, $oHostObject = null, $bLocalize = true)
	{
		// Display attribute as bar graph. Value & colors are provided by object holding attribute. 
		$iWidth = 5; // Total width of the percentage bar graph, in em...
		if ($oHostObject != null)
		{
			$aParams = array();
			$aParams = $oHostObject->GetAttributeParams($this->GetCode());
			$sValue = $aParams ['value'];
			$sColor = $aParams ['color'];
		}
		else
		{
			$sValue = 0;
			$sColor = GREEN;
		}
		$iValue = (int)$sValue;
		$iPercentWidth = ($iWidth * $iValue) / 100;
		return "<div style=\"width:{$iWidth}em;-moz-border-radius: 3px;-webkit-border-radius: 3px;border-radius: 3px;display:inline-block;border: 1px #ccc solid;\"><div style=\"width:{$iPercentWidth}em; display:inline-block;background-color:$sColor;\">&nbsp;</div></div>&nbsp;$sValue %";
	}

	public function GetAsCSV($sValue, $sSeparator = ',', $sTextQualifier = '"', $oHostObject = null, $bLocalize = true)
	{
		if ($oHostObject != null)
		{
			$aParams = array();
			$aParams = $oHostObject->GetAttributeParams($this->GetCode());
			$sValue = $aParams ['value'];
		}
		else
		{
			$sValue = 0;
		}
		//$sEscaped = (string)mylong2ip($sValue);
		$sEscaped = (string)$sValue;
		return $sTextQualifier.$sEscaped.$sTextQualifier;
	}
}

/**************************************
 * Functions to handle masks and sizes
 */

function MaskToSize($Mask)
{
	// Provides size of subnet according to dotted string mask 
	switch ($Mask)
	{
		case "0.0.0.0": return 4294967296;
		case "128.0.0.0": return 2147483648;
		case "192.0.0.0": return 1073741824;
		case "224.0.0.0": return 536870912;
		case "240.0.0.0": return 268435456;
		case "248.0.0.0": return 134217728;
		case "252.0.0.0": return 67108864;
		case "254.0.0.0": return 33554432;
		case "255.0.0.0": return 16777216;
		case "255.128.0.0": return 8388608;
		case "255.192.0.0": return 4194304;
		case "255.224.0.0": return 2097152;
		case "255.240.0.0": return 1048576;
		case "255.248.0.0": return 524288;
		case "255.252.0.0": return 262144;
		case "255.254.0.0": return 131072;
		case "255.255.0.0": return 65536;
		case "255.255.128.0": return 32768;
		case "255.255.192.0": return 16384;
		case "255.255.224.0": return 8192;
		case "255.255.240.0": return 4096;
		case "255.255.248.0": return 2048;
		case "255.255.252.0": return 1024;
		case "255.255.254.0": return 512;
		case "255.255.255.0": return 256;
		case "255.255.255.128": return 128;
		case "255.255.255.192": return 64;
		case "255.255.255.224": return 32;
		case "255.255.255.240": return 16;
		case "255.255.255.248": return 8;
		case "255.255.255.252": return 4;
		case "255.255.255.254": return 2;
		case "255.255.255.255":
		default: return 1;
	}
}

function MaskToBit($Mask)
{
	// Provides number of bits within a dotted string mask
	return SizeToBit(MaskToSize($Mask));
}

function SizeToMask ($Size)
{
	// Convert size of subnet into mask
	if (($Size & ($Size - 1)) == 0)
	{
		//return (~($Size - 1));
		return (sprintf("%u", ~($Size - 1)));
	}
	else
	{
		return null;
	}
}

function SizeToBit($Size)
{
	// Provides number of bits for a given subnet size
	$bitMask = myip2long("128.0.0.0");
	$i = 1;
	//while(($Size < MaskToSize(mylong2ip($bitMask))) && ($i < 32))
	while(($Size < $bitMask) && ($i < 32))
	{
		$bitMask = $bitMask >> 1;
		$i++;
	}
	if ($i <= 32)  
	{
		return $i;
	}
	else
	{
		return 0;
	}
}

/**************************
 * Functions to handle IPs
 */
 
function myip2long($sIp)
{
	//return(($sIp == '255.255.255.255') ? MAX_IPV4_VALUE : ip2long($sIp)); // Doesn't work for IPs > 128.0.0.0
	return(($sIp == '255.255.255.255') ? MAX_IPV4_VALUE : sprintf("%u", ip2long($sIp))); // OK so far... 
} 

function mylong2ip ($iIp)
{
	return(long2ip($iIp));
}

function IpPings($sIp, $iTimeToWait)
{
	$sSystemType = strtoupper(php_uname($mode = "s"));
	if (strpos($sSystemType, 'WIN') === false)
	{
		// Unix type - what else?
		$sCommand = "ping -c ".NUMBER_OF_PINGS." -W ".($iTimeToWait*1000)." ".$sIp;
	}
	else
	{
		// Windows
		$sCommand = "ping -n ".NUMBER_OF_PINGS." -w ".($iTimeToWait*1000)." ".$sIp;
	}
	exec($sCommand, $aOutput, $iRes);
	if ($iRes == 0)
	{
		// IP pings
		$aOutput[0] = $sCommand;
		return $aOutput;  
	}
	else
	{
		return array();
	}
}

/******************************************
 * General Configuration related functions
 *  . function ReadGlobalIPConfig
 *  . function GetFromGlobalIPConfig
 *  . function DisplayGlobalParametersInLocalModifyForm  
 */

function ReadGlobalIPConfig($sOrgId)
{
	// Create Global Config of $sOrgId if it doesn't exist
	// Create basic IP usages at the same time
	$oIpConfig = MetaModel::GetObjectFromOQL("SELECT IPv4Config AS conf WHERE conf.org_id = $sOrgId", null, false);
	if ($oIpConfig == null)
	{
		$oMyChange = MetaModel::NewObject("CMDBChange");
		$oMyChange->Set("date", time());
		$sUserString = CMDBChange::GetCurrentUserName();
		$oMyChange->Set("userinfo", $sUserString);
		$iChangeId = $oMyChange->DBInsert();
		
		$oIpConfig = MetaModel::NewObject('IPv4Config');
		$oIpConfig->Set('org_id', $sOrgId);
		$oIpConfig->DBInsertTracked($oMyChange);
		
		CreateBasicIpUsage($sOrgId);
	}
	return ($oIpConfig);
}

function GetFromGlobalIPConfig($sParameter, $sOrgId)
{
	// Reads $sParameter from Global Config 
	if ($sOrgId != null)
	{
		$oIpConfig = ReadGlobalIPConfig($sOrgId);
		return ($oIpConfig->Get($sParameter));
	}
	return null;
}

function DisplayGlobalParametersInLocalModifyForm(WebPage $oP, $sParameter, $sOrgId, $aDefault = array())
{
	// Get Global config object
	$oIpConfig = ReadGlobalIPConfig($sOrgId);
	$aDetails = array();
	
	// Display Parameter with option to be changed for the transaction
	$i = 0;
	$sParam = $sParameter[$i++];
	while ($sParam != null)
	{
		$sInputId = $sParam;
		$oAttDef = MetaModel::GetAttributeDef('IPv4Config', $sParam);
		$sValue = (array_key_exists($sParam, $aDefault)) ? $aDefault[$sParam] : $oIpConfig->Get($sParam);
		$sDisplayValue = $oIpConfig->GetEditValue($sParam);
		$iFlags = $oIpConfig->GetAttributeFlags($sParam);
		$aArgs = array('this' => $oIpConfig, 'formPrefix' => '');
		$sHTMLValue = "<span id=\"field_{$sInputId}\">".$oIpConfig->GetFormElementForField($oP, 'IPv4Config', $sParam, $oAttDef, $sValue, $sDisplayValue, $sInputId, '', $iFlags, $aArgs).'</span>';
		$aDetails[] = array('label' => '<span title="'.$oAttDef->GetDescription().'">'.$oAttDef->GetLabel().'</span>', 'value' => $sHTMLValue);
		$sParam = $sParameter[$i++];
	}
	
	$oP->Details($aDetails);
}

function DisplayGlobalConfig(WebPage $oP, $sOperation, $sOrgId, $aDefault = array())
{
	switch ($sOperation)
	{
		case 'shrinkblock':
		case 'splitblock':
		case 'expandblock':
		default:
			$sLabelOfAction = Dict::Format('Class:IPv4Block/Tab:globalparam');
			$sParameter = array ('block_min_size', 'block_cidr_aligned', null);
		break;
	}
	
	$oP->SetCurrentTab($sLabelOfAction);
	$oP->p(Dict::Format('UI:IPManagement:Action:Change:GlobalConfig'));
	$oP->add('<table style="vertical-align:top"><tr>');
	$oP->add('<td style="vertical-align:top">');
	
	DisplayGlobalParametersInLocalModifyForm($oP, $sParameter, $sOrgId, $aDefault);      
	
	$oP->add('</td>');
	$oP->add('</tr></table>');
}

/**************************
 * Block related functions
 *  . function IsBlockCIDRAligned
 */

function IsBlockCIDRAligned($sOrgId, $iFirstIp, $iLastIp)
{
	$sBlockCidrAligned = utils::ReadPostedParam('attr_block_cidr_aligned', '');
	if (empty($sBlockCidrAligned))
	{
		$sBlockCidrAligned = GetFromGlobalIPConfig('block_cidr_aligned', $sOrgId);
	}
	if ($sBlockCidrAligned == 'bca_yes')
	{
		// Compute size of new block and check if it corresponds to size of a CIDR block
		$Size = $iLastIp - $iFirstIp + 1;
		if (($Size & ($Size - 1)) != 0)
		{
			return false;
		} 
		// Check that FirstIp is CIDR aligned
		// Call to ip2long(long2ip()) is a workaround to handle integers that are above their max size
		$iMask = SizeToMask($Size);
		if ((ip2long(long2ip($iFirstIp)) & ip2long(long2ip($iMask))) != ip2long(long2ip($iFirstIp)))
		{
			return false;
		}
	}
	return true;
}

/****************************
 * Subnets related functions
 *  . function IsSubnetCIDRAligned
 */

function IsSubnetCIDRAligned($iIp, $iMask)
{
	/**
	 * Checks if the subnet is aligned to CIDR borders
	 */
	if (($iIp == 0) && ($iMask == 0))
	{
		$iIp = myip2long($this->Get('ip'));
		$iMask = myip2long($this->Get('mask'));
	}
	
	// Check that FirstIp is CIDR aligned
	// Call to ip2long(long2ip()) is a workaround to handle integers that are above their max size
	if ((ip2long(long2ip($iIp)) & ip2long(long2ip($iMask))) != ip2long(long2ip($iIp)))
	{
		return false;
	}
	return true;
}

/*********************************
 * IP Addresses related functions
 *  . CreateBasicIpUsage
 *  . GetIpUsageId
 *  . IsFqdnUnique 
 */

function CreateBasicIpUsage($sOrgId)
{
	$bChangeInserted = false;
	$oIpUsage = MetaModel::GetObjectFromOQL("SELECT IPv4Usage AS i WHERE i.name = 'NETWORK_IP_CODE' AND i.org_id = $sOrgId", null, false);
	if ($oIpUsage == null)
	{
		$oMyChange = MetaModel::NewObject("CMDBChange");
		$oMyChange->Set("date", time());
		$sUserString = CMDBChange::GetCurrentUserName();
		$oMyChange->Set("userinfo", $sUserString);
		$iChangeId = $oMyChange->DBInsert();
		$bChangeInserted = true;
		
		$oIpUsage = MetaModel::NewObject('IPv4Usage');
		$oIpUsage->Set('org_id', $sOrgId);
		$oIpUsage->Set('name', NETWORK_IP_CODE);
		$oIpUsage->Set('description', NETWORK_IP_DESC);
		$oIpUsage->DBInsertTracked($oMyChange);
	}
	$oIpUsage = MetaModel::GetObjectFromOQL("SELECT IPv4Usage AS i WHERE i.name = 'BROADCAST_IP_CODE' AND i.org_id = $sOrgId", null, false);
	if ($oIpUsage == null)
	{
		if (!$bChangeInserted)
		{
			$oMyChange = MetaModel::NewObject("CMDBChange");
			$oMyChange->Set("date", time());
			$sUserString = CMDBChange::GetCurrentUserName();
			$oMyChange->Set("userinfo", $sUserString);
			$iChangeId = $oMyChange->DBInsert();
		}
		
		$oIpUsage = MetaModel::NewObject('IPv4Usage');
		$oIpUsage->Set('org_id', $sOrgId);
		$oIpUsage->Set('name', BROADCAST_IP_CODE);
		$oIpUsage->Set('description', BROADCAST_IP_DESC);
		$oIpUsage->DBInsertTracked($oMyChange);
	}
} 

function GetIpUsageId($sOrgId, $sCode)
{
	$oIpUsage = MetaModel::GetObjectFromOQL("SELECT IPv4Usage AS i WHERE i.name = '$sCode' AND i.org_id = $sOrgId", null, false);
	if ($oIpUsage == null)
	{
		$oMyChange = MetaModel::NewObject("CMDBChange");
		$oMyChange->Set("date", time());
		$sUserString = CMDBChange::GetCurrentUserName();
		$oMyChange->Set("userinfo", $sUserString);
		$iChangeId = $oMyChange->DBInsert();
		
		$oIpUsage = MetaModel::NewObject('IPv4Usage');
		$oIpUsage->Set('org_id', $sOrgId);
		$oIpUsage->Set('name', $sCode);
		if ($sCode == NETWORK_IP_CODE)
		{
			$sDesc = NETWORK_IP_DESC;
		}
		else
		{
			$sDesc = BROADCAST_IP_DESC;
		}
		$oIpUsage->Set('description', $sDesc);
		$oIpUsage->DBInsertTracked($oMyChange);
	}
	return ($oIpUsage->GetKey());
}

function IsFqdnUnique($sOrgId, $iIpKey, $sShortName, $sDomainName)
{
	// The check takes into account the global parameters that defines if duplicate FQDNs are authorized or not
	$sIpAllowDuplicateName = utils::ReadPostedParam('attr_ip_allow_duplicate_name', '');
	if (empty($sIpAllowDuplicateName))
	{
		$sIpAllowDuplicateName = GetFromGlobalIPConfig('ip_allow_duplicate_name', $sOrgId);
	}
	if ($sIpAllowDuplicateName == 'ipdup_no')
	{
		if ($sShortName != "")
		{
			$oIpSet = new CMDBObjectSet(DBObjectSearch::FromOQL("SELECT IPv4Address AS i WHERE i.short_name = '$sShortName' AND i.org_id = $sOrgId"));
			// Match for creations is verbiden. Match for modifications as well unless the current name is kept
			$sModifWithCurrentName = false;
			while ($oIp = $oIpSet->Fetch())
			{
				if ($iIpKey == $oIp->GetKey())
				{
					// One of the objects with same name has same ID -> it's a modification
					$sModifWithCurrentName = true;
				}
			}
			if (!$sModifWithCurrentName)
			{
				$oIpSet->Rewind();
				while ($oIp = $oIpSet->Fetch())
				{
					if ($iIpKey != $oIp->GetKey())
					{
						// Name of different objects are the same. 
						// Are domain names identical as well?
						$sOtherDomainName = $oIp->Get('domain_name');
						if ($sDomainName == $sOtherDomainName)
						{
							// Check status of IP before complaining - Released IPs don't count.
							if ($oIp->Get('status') != 'released')
							{
								return false;
							}
						}
					}
				}
			}
		}
	}
	return true;
}

/******************************
 * Triggers related to IP classes
 *  . IPTriggerOnWaterMark 
 */

class IPTriggerOnWaterMark extends Trigger
{
	public static function Init()
	{
		$aParams = array
		(
			"category" => "bizmodel",
			"key_type" => "autoincrement",
			"name_attcode" => "description",
			"state_attcode" => "",
			"reconc_keys" => array(),
			"db_table" => "priv_trigger_onwatermark",
			"db_key_field" => "id",
			"db_finalclass_field" => "",
			"display_template" => "",
			"icon" => utils::GetAbsoluteUrlModulesRoot().'teemip-ip-mgmt/images/ipbell.png',
		);
		MetaModel::Init_Params($aParams);
		MetaModel::Init_InheritAttributes();
		
		MetaModel::Init_AddAttribute(new AttributeExternalKey("org_id", array("targetclass"=>"Organization", "jointype"=>null, "allowed_values"=>null, "sql"=>"org_id", "is_null_allowed"=>false, "on_target_delete"=>DEL_MANUAL, "depends_on"=>array())));
		MetaModel::Init_AddAttribute(new AttributeClass("target_class", array("class_category"=>"ipmgmt", "more_values"=>null, "sql"=>"target_class", "default_value"=>null, "is_null_allowed"=>false, "depends_on"=>array())));
		MetaModel::Init_AddAttribute(new AttributeEnum("event", array("allowed_values"=>new ValueSetEnum('cross_high,cross_low'), "sql"=>"event", "default_value"=>"cross_high", "is_null_allowed"=>true, "depends_on"=>array())));
		
		// Display lists
		MetaModel::Init_SetZListItems('details', array('org_id', 'description', 'target_class', 'event', 'action_list')); // Attributes to be displayed for the complete details
		MetaModel::Init_SetZListItems('list', array('finalclass', 'target_class', 'description', 'event', 'org_id')); // Attributes to be displayed for a list
	}
}

/***********************************
 * Plugin to extend the Popup Menus
 */

class IPMgmtExtraMenus implements iPopupMenuExtension
{
	public static function EnumItems($iMenuId, $param)
	{
		switch($iMenuId)
		{
			case iPopupMenuExtension::MENU_OBJLIST_ACTIONS:	// $param is a DBObjectSet
				$oSet = $param;
				if ($oSet->Count() == 1)
				{
					// Menu for single objects only 
					$oObj = $oSet->Fetch();
					
					// Additional actions for IPv4Blocks
					if ($oObj instanceof IPv4Block)
					{
						$oAppContext = new ApplicationContext();
						$sContext = $oAppContext->GetForLink();
						
						// Unique org is selected as we have a single object
						$id = $oObj->GetKey();
						$sOrgId = $oObj->Get('org_id');
						
						$operation = utils::ReadParam('operation', '');
						switch ($operation)
						{
							case 'displaytree':
								if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Block', Dict::S('UI:IPManagement:Action:Shrink:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinkblock&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Block', Dict::S('UI:IPManagement:Action:Split:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitblock&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Block', Dict::S('UI:IPManagement:Action:Expand:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandblock&class=IPv4Block&id=$id&$sContext"),
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Block', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Block&id=$id&$sContext"),
									);
								}
								else
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext"),
									);
								}
							break;
										
							case 'displaylist':
							default:
								if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Block', Dict::S('UI:IPManagement:Action:Shrink:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinkblock&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Block', Dict::S('UI:IPManagement:Action:Split:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitblock&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Block', Dict::S('UI:IPManagement:Action:Expand:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandblock&class=IPv4Block&id=$id&$sContext"),
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Block', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Block&id=$id&$sContext"),
									);
								}
								else
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext"),
									);
								}
							break;
						}
					}
					// Additional actions for IPv4Subnets
					elseif ($oObj instanceof IPv4Subnet)
					{
						$oAppContext = new ApplicationContext();
						$sContext = $oAppContext->GetForLink();
						
						// Unique org is selected as we have a single object
						$id = $oObj->GetKey();
						$sOrgId = $oObj->Get('org_id');
						
						$operation = utils::ReadParam('operation', '');
						switch ($operation)
						{
							case 'displaytree':
								if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Subnet', Dict::S('UI:IPManagement:Action:Shrink:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinksubnet&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Subnet', Dict::S('UI:IPManagement:Action:Split:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitsubnet&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Subnet', Dict::S('UI:IPManagement:Action:Expand:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandsubnet&class=IPv4Subnet&id=$id&$sContext"),
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext"),
									);
								}
								else
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext"),
									);
								}
							break;
										
							case 'displaylist':
							default:
								if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Subnet', Dict::S('UI:IPManagement:Action:Shrink:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinksubnet&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Subnet', Dict::S('UI:IPManagement:Action:Split:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitsubnet&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Subnet', Dict::S('UI:IPManagement:Action:Expand:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandsubnet&class=IPv4Subnet&id=$id&$sContext"),
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext"),
									);
								}
								else
								{
									$aResult = array(
										new SeparatorPopupMenuItem(),
										new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext"),
										new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext"),
									);
								}
							break;
						}
					}
					else
					{
						$aResult = array();
					}
				}
				else
				{
					$aResult = array();
				}
			break;
			
			case iPopupMenuExtension::MENU_OBJLIST_TOOLKIT: // $param is a DBObjectSet
				$oSet = $param;
				$oObj = $oSet->Fetch();
				
				// Additional actions for IPv4Blocks
				if ($oObj instanceof IPv4Block)
				{
					$oAppContext = new ApplicationContext();
					$sContext = $oAppContext->GetForLink();
					
					// Check if unique org is selected or not
					$UniqueOrg = true;
					$sOrgId = $oObj->Get('org_id');
					while (($oBlock = $oSet->Fetch()) && $UniqueOrg)
					{
						if ($oBlock->Get('org_id') != $sOrgId)
						{
							$sOrgId = ALL_ORGS;
							$UniqueOrg = false;			     
						}
					}
					
					$operation = utils::ReadParam('operation', '');
					switch ($operation)
					{
						case 'displaytree':
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:DisplayList:IPv4Block', Dict::S('UI:IPManagement:Action:DisplayList:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=displaylist&class=IPv4Block&org_id=$sOrgId&$sContext"),
							);
						break;
										
						case 'displaylist':
						default:
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:DisplayTree:IPv4Block', Dict::S('UI:IPManagement:Action:DisplayTree:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=displaytree&class=IPv4Block&org_id=$sOrgId&$sContext"),
							);
						break;
					}
				}
				// Additional actions for IPv4Subnets
				else if ($oObj instanceof IPv4Subnet)
				{
					$oAppContext = new ApplicationContext();
					$sContext = $oAppContext->GetForLink();
					
					// Check if unique org is selected or not
					$UniqueOrg = true;
					$sOrgId = $oObj->Get('org_id');
					while (($oBlock = $oSet->Fetch()) && $UniqueOrg)
					{
						if ($oBlock->Get('org_id') != $sOrgId)
						{
							$sOrgId = ALL_ORGS;
							$UniqueOrg = false;			     
						}
					}
					
					$operation = utils::ReadParam('operation', '');
					switch ($operation)
					{
						case 'displaytree':
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:DisplayList:IPv4Subnet', Dict::S('UI:IPManagement:Action:DisplayList:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=displaylist&class=IPv4Subnet&org_id=$sOrgId&$sContext"),
							);
						break;
										
						case 'displaylist':
						default:
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:DisplayTree:IPv4Subnet', Dict::S('UI:IPManagement:Action:DisplayTree:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=displaytree&class=IPv4Subnet&org_id=$sOrgId&$sContext"),
							);
						break;
					}
				}
				else
				{
					$aResult = array();
				}
			break;
			
			case iPopupMenuExtension::MENU_OBJDETAILS_ACTIONS: // $param is a DBObject
				$oObj = $param;
				
				// Additional actions for IPv4Blocks
				if ($oObj instanceof IPv4Block)
				{
					$oAppContext = new ApplicationContext();
					$sContext = $oAppContext->GetForLink();
					$id = $oObj->GetKey();
					$sOrgId = $oObj->Get('org_id');
					
					if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
					{
						$aResult = array(
							new SeparatorPopupMenuItem(),
							new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Block', Dict::S('UI:IPManagement:Action:Shrink:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinkblock&class=IPv4Block&id=$id&$sContext"),
							new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Block', Dict::S('UI:IPManagement:Action:Split:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitblock&class=IPv4Block&id=$id&$sContext"),
							new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Block', Dict::S('UI:IPManagement:Action:Expand:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandblock&class=IPv4Block&id=$id&$sContext"),
							new SeparatorPopupMenuItem(),
						);
					}
					else
					{
						$aResult = array();
					}
					$operation = utils::ReadParam('operation', '');
					switch ($operation)
					{
						case 'apply_new':
						case 'apply_modify':
						case 'details':
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext");
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Block', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Block&id=$id&$sContext");
							}
						break;
						
						case 'listspace':
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Block', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Block&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Block', Dict::S('UI:IPManagement:Action:Details:IPv4Block'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Block&id=$id&$sContext");
						break;
						
						case 'dofindspace':
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:ListSpace:IPv4Block', Dict::S('UI:IPManagement:Action:ListSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listspace&class=IPv4Block&id=$id&$sContext");
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Block', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Block'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Block&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Block', Dict::S('UI:IPManagement:Action:Details:IPv4Block'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Block&id=$id&$sContext");
						break;
						
						default:
						break;
					}
				}
				// Additional actions for IPv4Subnets
				else if ($oObj instanceof IPv4Subnet)
				{
					$oAppContext = new ApplicationContext();
					$sContext = $oAppContext->GetForLink();
					$id = $oObj->GetKey();
					$sOrgId = $oObj->Get('org_id');
					
					if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
					{
						$aResult = array(
							new SeparatorPopupMenuItem(),
							new URLPopupMenuItem('UI:IPManagement:Action:Shrink:IPv4Subnet', Dict::S('UI:IPManagement:Action:Shrink:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=shrinksubnet&class=IPv4Subnet&id=$id&$sContext"),
							new URLPopupMenuItem('UI:IPManagement:Action:Split:IPv4Subnet', Dict::S('UI:IPManagement:Action:Split:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=splitsubnet&class=IPv4Subnet&id=$id&$sContext"),
							new URLPopupMenuItem('UI:IPManagement:Action:Expand:IPv4Subnet', Dict::S('UI:IPManagement:Action:Expand:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=expandsubnet&class=IPv4Subnet&id=$id&$sContext"),
							new SeparatorPopupMenuItem(),
						);
					}
					else
					{
						$aResult = array();
					}
					$operation = utils::ReadParam('operation', '');
					switch ($operation)
					{
						case 'apply_new':
						case 'apply_modify':
						case 'details':
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext");
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext");
						break;
						
						case 'listips':
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Subnet', Dict::S('UI:IPManagement:Action:Details:IPv4Subnet'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Subnet&id=$id&$sContext");
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext");
						break;
						
						case 'dofindspace':
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext");
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Subnet', Dict::S('UI:IPManagement:Action:Details:IPv4Subnet'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Subnet&id=$id&$sContext");
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Subnet&id=$id&$sContext");
						break;
						
						case 'csvexportips':
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Subnet', Dict::S('UI:IPManagement:Action:ListIps:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Subnet&id=$id&$sContext");
							if (UserRights::IsActionAllowed('IPv4Block', UR_ACTION_MODIFY) == UR_ALLOWED_YES)
							{
								$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:FindSpace:IPv4Subnet', Dict::S('UI:IPManagement:Action:FindSpace:IPv4Subnet'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=findspace&class=IPv4Subnet&id=$id&$sContext");
							}
							$aResult[] = new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Subnet', Dict::S('UI:IPManagement:Action:Details:IPv4Subnet'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Subnet&id=$id&$sContext");
						break;
						
						default:
						break;
					}
				}
				// Additional actions for IPv4Range
				else if ($oObj instanceof IPv4Range)
				{
					$oAppContext = new ApplicationContext();
					$sContext = $oAppContext->GetForLink();
					$id = $oObj->GetKey();
					
					$operation = utils::ReadParam('operation', '');
					switch ($operation)
					{
						case 'listips':
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Range', Dict::S('UI:IPManagement:Action:Details:IPv4Range'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Range&id=$id&$sContext"),
								new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Range', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Range'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Range&id=$id&$sContext"),
							);
							break;
							
						case 'csvexportips':
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Range', Dict::S('UI:IPManagement:Action:ListIps:IPv4Range'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Range&id=$id&$sContext"),
								new URLPopupMenuItem('UI:IPManagement:Action:Details:IPv4Range', Dict::S('UI:IPManagement:Action:Details:IPv4Range'), utils::GetAbsoluteUrlAppRoot()."pages/UI.php?operation=details&class=IPv4Range&id=$id&$sContext"),
							);
							break;
							
						default:
							$aResult = array(
								new SeparatorPopupMenuItem(),
								new URLPopupMenuItem('UI:IPManagement:Action:ListIps:IPv4Range', Dict::S('UI:IPManagement:Action:ListIps:IPv4Range'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=listips&class=IPv4Range&id=$id&$sContext"),
								new URLPopupMenuItem('UI:IPManagement:Action:CsvExportIps:IPv4Range', Dict::S('UI:IPManagement:Action:CsvExportIps:IPv4Range'), utils::GetAbsoluteUrlModulesRoot()."teemip-ip-mgmt/ui.teemip-ip-mgmt.php?operation=csvexportips&class=IPv4Range&id=$id&$sContext"),
							);
							break;
					}
				}
				else
				{
					$aResult = array();
				}
			break;
			
			case iPopupMenuExtension::MENU_DASHBOARD_ACTIONS:
				// $param is a Dashboard
				$aResult = array();
				break;
			
			default:
				// Unknown type of menu, do nothing
				$aResult = array();
				break;
		}
		return $aResult;
	}
}

?>